<?php

namespace App\Http\Controllers;

use DB;
use App\Device;
use App\Setting;
use App\DeviceAd;
use App\UserDevice;
use App\Http\Requests;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Mail;


class DeviceController extends Controller
{
    public function __construct()
    {
    	//DI container
    }

    public function settings()
    {
        $settings = Setting::all();
        $ar = [];
        if(count($settings))
        {
            foreach($settings AS $s)
            {
                $ar[$s->key] = $s->value;
            }
        }
        return response()->json($ar,200);
    }

    public function device($device_id)
    {
         $device = UserDevice::where('device_id_string', $device_id)->first();

        if($device==null)
        {
            return response()->json(['success'=>false], 400);
        }

        $device->portrait_logo = 'http://'.$_SERVER['HTTP_HOST'].$device->portrait_logo;

        $device->kiosk_mode = (bool) $device->kiosk_mode;
        $device->queue_mode = (bool) $device->queue_mode;

        return response()->json($device, 200);

    }

    public function toggle($device_id)
    {
         $device = UserDevice::where('device_id_string', $device_id)->first();

        if($device==null)
        {
            return response()->json(['success'=>false], 400);
        }

        $device->queue_mode = $device->queue_mode==false ? true : false;
        $device->save();

        return response()->json($device, 200);

    }


    public function playlist($device_id)
    {
        $device = UserDevice::where('device_id_string', $device_id)->first();

        if($device == null)
        {
            return response()->json(['success' => false], 400);
        }

        $device->last_checked = date('Y-m-d G:i:s');
        $device->save();

        $uploads = DB::table('uploads')
            ->join('device_ads', 'uploads.id', '=', 'device_ads.upload_id')
            ->where('device_ads.device_id', $device->device_id)
            ->where('uploads.is_ready', 1)
            ->select('uploads.*', 'device_ads.start_time', 'device_ads.end_time','device_ads.position AS position')
            ->get();

        $data = [];
        if(count($uploads)>0)
        {
            foreach($uploads AS $u)
            {
                $ob = new \stdClass;
                $ob->id = $u->id;
                $ob->location = 'http://'.$_SERVER['HTTP_HOST'].$u->location;
                $ob->thumb_location = 'http://'.$_SERVER['HTTP_HOST'].$u->thumb_location;
                $ob->type = $u->type==1 ? 'image' : 'video';
                $ob->content_type = $u->content_type;
                $ob->name = $u->name;
                $ob->created_at = $u->created_at;
                $ob->updated_at = $u->updated_at;
                $ob->start_time =  ($u->start_time==null || trim(strtolower($u->start_time))=='null') ? null : date('Y-m-d H:i:s', strtotime($u->start_time));
                $ob->end_time = ($u->end_time==null || trim(strtolower($u->end_time))=='null')? null : date('Y-m-d H:i:s', strtotime($u->end_time));
                $ob->length = $u->length;
                $ob->position = $u->position;
                $info = pathinfo($u->location);
                $ob->extension = $info['extension'];

                if($u->type==2)
                {
                    $time = $u->length;
                    $dt = new \DateTime("1970-01-01 $time", new \DateTimeZone('UTC'));
                    $seconds = (int) $dt->getTimestamp();

                    $ob->duration_sconds = $seconds;
                }
                else
                {
                    $setting = Setting::where('key', 'DEFAULT_IMAGE_DURATION')->first();
                    $ob->duration_seconds = $setting==null ? 30 : $setting->value; 
                }


                $data[] = $ob;
            }

        }
        return response()->json($data, 200);


    }

    public function code($code)
    {
        return Device::where('code', $code)->first();
    }

}
