<?php

namespace App\Http\Controllers;

use DB;
use Curl;
use App\Device;
use App\Upload;
use App\Setting;
use App\DeviceAd;
use App\UserDevice;
use App\Http\Requests;
use App\Mail\DeviceOffline;
use Illuminate\Http\Request;
use App\Mail\DeviceActivated;
use Illuminate\Http\Response;
use App\Mail\DeviceDeactivated;
use App\Jobs\SendPushNotification;
use Illuminate\Support\Facades\Mail;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('home');
    }

    public function priority()
    {
        return view('priority');
    }

    public function screen($id, Request $request)
    {
        $screen = UserDevice::where('device_id', $id)
                    // ->where('user_id', $request->user()->id)
                    ->first();
        if ($screen == null) {
            return ["not today"];
        }
        return view('screen', ["screen" => $screen]);
    }

    public function toggleKiosk($id, Request $request)
    {
        $screen = UserDevice::where('device_id', $id)
                    // ->where('user_id', $request->user()->id)
                    ->first();

        $screen->kiosk_mode = $screen->kiosk_mode==false ? true : false;
        $screen->save();

        return $screen;
    }

    public function updateSettings(Request $request)
    {
        $data = $request->only('image_duration');

        $setting = Setting::where('key', 'DEFAULT_IMAGE_DURATION')->first();

        if ($setting==true) {
            $setting->value=$data['image_duration'];
            $setting->save();
        } else {
            $setting = new Setting;
            $setting->key = 'DEFAULT_IMAGE_DURATION';
            $setting->value=$data['image_duration'];
            $setting->save();
        }

        return response()->json($setting, 200);
    }

    public function settings()
    {
        return response()->json(Setting::all(), 200);
    }


    public function playback($id, Request $request)
    {
        $screen = UserDevice::where('device_id', $id)
                    // ->where('user_id', $request->user()->id)
                    ->first();
        if ($screen == null) {
            return ["not today"];
        }

        $uploads = DB::table('uploads')
            ->join('device_ads', 'uploads.id', '=', 'device_ads.upload_id')
            ->where('device_ads.device_id', $id)
            ->select('uploads.*', 'device_ads.start_time', 'device_ads.end_time')
            ->get();

        return view('playback', ["media"=>$uploads]);
    }



    public function upload()
    {
        return view('upload');
    }


    public function studio()
    {
        return view('studio');
    }

    public function template($id=null)
    {
        $data = [];
        $data['mode'] = $id==null ? 'new' : 'edit';
        return view('template', $data);
    }

    public function library(Request $request)
    {
        $fetch = Upload::/*where('user_id', $request->user()->id)->*/orderBy('updated_at', 'desc')->get();

        $data['library'] = $fetch;

        return view('library', $data);
    }

    public function add(Request $request)
    {
        $data = $request->only('name', 'code', 'email');

        if ($data['name'] == '' || $data['code'] == '') {
            return response()->json(['success' => false, 'error' => 'Please fill in all fields'], 400);
        }

        $tenant = explode(".", str_replace("wwww.", '', $_SERVER['HTTP_HOST']));
        $tenant_name = count($tenant)>2 ? $tenant[0] : 'test';

        $url = env('API_URL', 'https://api.digify.tv/device/update_tenant');

        $response = Curl::to($url)
        ->withData(array( 'tenant' => $tenant_name, 'code'=>$data['code'] ))
        ->post();

        if ($response==false) {
            return response()->json(['success'=>false, 'error'=>'Device not recognized by system'], 400);
        }

        $res = json_decode($response);

        \Log::info($response);

        if ($res->device->tenant == $tenant_name) {
            //take this device
            $user_device = UserDevice::where('device_id_string', $res->device->device_id)->first();
            if ($user_device==null) {
                $device = new Device;
                $device->name = $data['name'];
                $device->code = $data['code'];
                $device->device_id = $res->device->device_id;
                $device->status = 1;
                $device->save();

                //add this device to the user devices
                $user_device = new UserDevice;
                $user_device->device_id_string = $device->device_id;
                $user_device->device_id = $device->id;
                $user_device->user_id = $request->user()->id;
                $user_device->name = $data['name'];
                $user_device->email = $data['email'] ?? null;
            }

            if ($user_device->save()) {
                /* $obj = new \stdClass;
                 $obj->notificationType = "AssignmentSuccessful";
                 $obj->deviceToken = $res->device->push_id;
                 $obj->device_id = $res->device->device_id;
                 $this->dispatch(new SendPushNotification($obj));*/
            }

            //dispatch push notification

            return response()->json($user_device, 200);
        }

        return response()->json(['success'=>false, 'error'=>"Could not add screen"], 400);
    }

    public function screens(Request $request)
    {
        return UserDevice::/*where('user_id', $request->user()->id)->*/orderBy('updated_at', 'desc')->get();
    }

    public function updateUserScreen($id, Request $request)
    {
        if ($request['name']=='' || $request['code'] == '') {
            return response()->json(['success'=>false, 'error'=>'Please fill in all fields'], 400);
        }

        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        if ($screen == null) {
            return response()->json(['success' => false, 'error' => 'Device not found'], 400);
        }

        $screen->name = $request['name'];
        $screen->email = $request['email'];
        $screen->save();

        return response()->json($screen, 200);
    }


    public function updateUserScreenPosition($id, Request $request)
    {
        $data  = $request->only('data');
        if (count($data['data'])==0) {
            return response()->json(['success'=>false, 'error'=>'Send one or more ids'], 400);
        }

        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        if ($screen==null) {
            return response()->json(['success'=>false, 'error'=>'Device not found'], 400);
        }

        $position = 0;
        foreach ($data['data'] as $a) {
            $media = DeviceAd::where('device_id', $screen->device_id)->where('id', $a)->first();
            $media->position = $position;
            $media->save();
            $position = $position + 1;
        }

        /*  $obj = new \stdClass;
          $obj->notificationType = "PositionChanged";
          $obj->deviceToken = Device::find($screen->device_id)->push_id;
          $obj->device_id = $screen->device_id_string;
           $this->dispatch(new SendPushNotification($obj));*/

        return response()->json($screen, 200);
    }

    public function updateUserScreenMode($id, Request $request)
    {
        $data  = $request->only('mode');

        if ($data['mode']!='portrait' && $data['mode']!='landscape') {
            return response()->json(['success'=>false, 'error'=>'portrait or landscape'], 400);
        }
        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        if ($screen==null) {
            return response()->json(['success'=>false, 'error'=>'Device not found'], 400);
        }

        $screen->mode = $data['mode'];
        $screen->save();

        return response()->json($screen, 200);
    }

    public function copyScreen($id, Request $request)
    {
        $data  = $request->only('data');
        if (count($data['data'])==0) {
            return response()->json(['success'=>false, 'error'=>'Send one or more ids'], 400);
        }
        $screen = UserDevice::where('device_id', $id)->first();
        if ($screen==null) {
            return response()->json(['success'=>false, 'error'=>'Device not found'], 400);
        }

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/



        foreach ($data['data'] as $a) {
            //check if user own screen
            //delete screen  ads
            //get this screen ads
            //insert screen ads for sent screens

            $user_screen  = UserDevice::/*where('user_id', $request->user()->id)
                                       ->*/where('id', $a)->first();


            if ($user_screen!=null) {
                //copy mode etc

                $user_screen->mode = $screen->mode;
                $user_screen->portrait_logo = $screen->portrait_logo;
                $user_screen->portrait_message = $screen->portrait_message;
                $user_screen->save();

                DeviceAd::where('device_id', $user_screen->device_id)->delete();
                $fetch = DeviceAd::where('device_id', $id)->get();

                if (count($fetch)>0) {
                    foreach ($fetch as $f) {
                        $ad = new DeviceAd;
                        $ad->upload_id = $f->upload_id;
                        $ad->device_id = $user_screen->device_id;
                        $ad->start_time = $f->start_time;
                        $ad->end_time = $f->end_time;
                        $ad->position = $f->position;
                        $ad->save();
                    }
                }
            }
        }

        return response()->json($screen, 200);
    }


    public function saveTime(Request $request)
    {
        //send global
        //send end_time, start_time
        //send ad_id (must use if not set to global)
        //send upload_id (must use if set to global)


        $data = $request->only('global', 'end_time', 'start_time', 'ad_id', 'upload_id');


        $upload = Upload::find($data['upload_id']);

        /*if ($upload==null || $upload->user_id!=$request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        $ad = DeviceAd::find($data['ad_id']);

        /*if ($ad->upload_id!=$upload->id) {
            return response()->json(['success'=>false, 'error'=>'Not your upload'], 400);
        }*/

        if ($data['global']!=null && $data['global']==true) {
            DeviceAd::where('upload_id', $upload->id)->update(array('start_time'=>$data['start_time'], 'end_time'=>$data['end_time']));
        } else {
            DeviceAd::where('id', $data['ad_id'])->update(array('start_time'=>$data['start_time'], 'end_time'=>$data['end_time']));
        }


        return response()->json($upload, 200);
    }


    public function deleteAdScreen($id, Request $request)
    {
        $data  = $request->only('ad_id', 'all');

        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        if ($screen == null) {
            return response()->json(['success' => false, 'error' => 'Device not found'], 400);
        }

        if ($data['all']==true) {
            //delete all
            $del = DeviceAd::where('device_id', $screen->device_id)->delete();
        } else {
            $del = DeviceAd::where('device_id', $screen->device_id)->where('id', $data['ad_id'])->delete();
        }
        /*
                    $obj = new \stdClass;
                    $obj->notificationType = "MediaDeleted";
                    $obj->deviceToken = Device::find($screen->device_id)->push_id;
                    $obj->device_id = $screen->device_id_string;
                     $this->dispatch(new SendPushNotification($obj)); */

        return response()->json($screen, 200);
    }

    public function deleteScreen()
    {
        $device_id = (int) request()->route('id');

        $device = UserDevice::where('id', '=', $device_id)->delete();

        if ($device == true) {
            return response()->json("success", 200);
        } elseif ($device == false) {
            return response()->json("failure", 422);
        }
    }

    public function activateScreen(Request $request)
    {
        $userDevice = UserDevice::where('id', '=', $request->screen_id)->first();

        Device::where('id', '=', $userDevice->device_id)->update(['status' => true]);

        if (! is_null($userDevice->email)) {
            Mail::to($userDevice->email)->send(new DeviceActivated($userDevice));
        }

        return response()->json("success", 200);
    }

    public function deactivateScreen(Request $request)
    {
        $userDevice = UserDevice::where('id', '=', $request->screen_id)->first();

        Device::where('id', '=', $userDevice->device_id)->update(['status' => false]);

        if (! is_null($userDevice->email)) {
            Mail::to($userDevice->email)->send(new DeviceDeactivated($userDevice));
        }

        return response()->json("success", 200);
    }

    public function uploadPortraitImage($id, Request $request)
    {
        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        $file = $request->file('file');

        if ($file==null) {
            return response()->json(['success'=>false, 'error'=>'File wrong'], 400);
        }
        $mime = $file->getMimeType();

        if (!preg_match("/image/is", $mime)) {
            return response()->json(['success'=>false, 'error'=>'Upload only image files'], 400);
        }

        $dir = public_path().'/uploads/'.date("Y/m/d").'/';
        if (!is_dir($dir)) {
            mkdir($dir, 0777, true);
        }

        $hash = md5_file($file->getRealPath());

        $extension = $file->getClientOriginalExtension();

        $hash_name = $hash.'.'.$extension;

        $location = '/uploads/'.date("Y/m/d").'/'.$hash_name;

        if ($file->move($dir, $hash_name)) {
            $screen->portrait_logo = $location;
            $screen->save();
        } else {
            return response()->json(['success'=>false, 'error'=>'Could not upload file'], 400);
        }

        return response()->json($screen, 200);
    }


    public function updatePortraitMessage($id, Request $request)
    {
        $data = $request->only('message');
        $screen = UserDevice::where('device_id', $id)->first();

        /*if ($screen->user_id != $request->user()->id) {
            return response()->json(['success'=>false, 'error'=>'Not your screen'], 400);
        }*/

        if ($data['message']=='') {
            return response()->json(['success'=>false, 'error'=>'Bad message'], 400);
        }

        $screen->portrait_message = $data['message'];

        $screen->save();

        return response()->json($screen, 200);
    }


    public function updateUploadInfo($upload_id, Request $request)
    {
        $data = $request->only("name", 'start_time', 'end_time');

        if ($data['name']=='') {
            return response()->json(['success'=>false, 'error'=>'Please fill in all fields'], 400);
        }

        $upload = Upload::/*where('user_id', $request->user()->id)
                ->*/where('id', $upload_id)
                ->first();

        if ($upload==null) {
            return response()->json(['success'=>false, 'error'=>'Not your upload'], 400);
        }


        $upload->name = $data['name'];
        $upload->start_time = $data['start_time'];
        $upload->end_time = $data['end_time'];

        DeviceAd::where('upload_id', $upload_id)->update(array('start_time'=>$data['start_time'], 'end_time'=>$data['end_time']));

        $upload->save();

        return response()->json($upload, 200);
    }


    public function addUploadToScreen($upload_id, Request $request)
    {
        $upload = Upload::/*where('user_id', $request->user()->id)
                ->*/where('id', $upload_id)
                ->first();

        if ($upload==null) {
            return response()->json(['success'=>false, 'error'=>'Not your upload'], 400);
        }

        $data = $request->only("screens", "start_date", "end_date");

        if (is_array($data['screens']) && !empty($data['screens']) && $data['screens']!=null) {
            foreach ($data['screens'] as $scr) {
                /*$check = DeviceAd::where('upload_id', $upload->id)->where("device_id", $scr['device_id'])->first();
                if($check==null)
                {*/
                $deviceAd = new DeviceAd;
                $deviceAd->upload_id = $upload->id;
                $deviceAd->device_id = $scr['device_id'];

                if ($data['start_date']!=null) {
                    $deviceAd->start_time = $data['start_date'];
                } else {
                    if ($upload->start_time!=null) {
                        $deviceAd->start_time = $upload->start_time;
                    }
                }

                if ($data['end_date']!=null) {
                    $deviceAd->end_time = $data['end_date'];
                } else {
                    if ($upload->end_time!=null) {
                        $deviceAd->end_time = $upload->end_time;
                    }
                }

                if ($deviceAd->save()) {
                    $deviceAd->position = $deviceAd->id;
                    $deviceAd->save();
                    /*  $obj = new \stdClass;
                      $obj->notificationType = "MediaAdded";
                      $obj->deviceToken = Device::find($scr['device_id'])->push_id;
                      $obj->device_id = $scr['device_id_string'];
                      $obj->upload = $upload;*/
                    /*  $this->dispatch(new SendPushNotification($obj));*/

                    $upload->screen_count = $upload->screen_count+1;
                }
                //}
            }
            $upload->save();
            $upload->ad_id = $deviceAd->id;
        }

        return response()->json(['success'=>true, 'message'=>"screens saved",'upload'=>$upload], 200);
    }


    public function deleteUpload($upload_id, Request $request)
    {
        $upload = Upload::where('user_id', $request->user()->id)
                ->where('id', $upload_id)
                ->first();

        if ($upload==null) {
            return response()->json(['success'=>false, 'error'=>'Not your upload'], 400);
        }

        $upload->delete();

        return response()->json(['success'=>true, 'message'=>"deleted"], 200);
    }


    public function dashboard()
    {
        $data['videos'] = Upload::where('type', 2)->count();
        $data['images'] = Upload::where('type', 1)->count();
        $data['files'] = $data['videos'] + $data['images'];
        // $data['disk'] = disk_free_space("/");

        $base = log(Upload::sum('size'), 1024);
        $suffixes = array('', 'KB', 'MB', 'GB', 'TB');

        $result =  round(pow(1024, $base - floor($base)), 2) .' '. $suffixes[floor($base)];
        $data['disk_used'] = $result;

        $data['devices'] = UserDevice::count();

        $data['offline'] = UserDevice::where('last_checked', '<', \Carbon\Carbon::now()->subHours(1))->count();

        $data['offline_devices'] = UserDevice::where('last_checked', '<', \Carbon\Carbon::now()->subHours(1))->get();

        $data['online_devices'] = UserDevice::where('last_checked', '>', \Carbon\Carbon::now()->subHours(1))->get();

        // Notifiy user when devices go offline.
        /*UserDevice::where('last_checked', '<', \Carbon\Carbon::now()
                    ->subMinutes(1))
                    ->get()
                    ->each(function($device) {
                        if (! is_null($device->email)) {
                            Mail::to($device->email)->send(new DeviceOffline($device));
                        }
                    });*/

        $data['online'] = $data['devices'] - $data['offline'];


        return view('dashboard', $data);
    }
}
