<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| This file is where you may define all of the routes that are handled
| by your application. Just tell Laravel the URIs it should respond
| to using a Closure or controller method. Build something great!
|
*/

Route::get('/', function () {
    return redirect()->to('/home');
});

Route::group(['prefix'=>'api'], function(){

	Route::get('device/playlist/{device_id}', 'DeviceController@playlist');
	Route::get('device/queue_mode/toggle/{device_id}', 'DeviceController@toggle');
	Route::get('device/{device_id}', 'DeviceController@device');
	Route::get('device/bycode/{code}', 'DeviceController@code');
	Route::post('device/push_id/{device_id}', 'DeviceController@updatePushId');
	Route::get('settings', 'DeviceController@settings');

});

Auth::routes();

Route::get('/home', 'HomeController@dashboard');
Route::get('/screens', 'HomeController@index');
Route::get('/priority', 'HomeController@priority');
Route::get('/studio', 'HomeController@studio');
Route::get('/template', 'HomeController@template');
Route::get('/template/{id}', 'HomeController@template');
Route::get('/screen/{id}', 'HomeController@screen');
Route::get('/playback/{id}', 'HomeController@playback');

Route::post('/upload', 'UploadController@doUpload');
Route::post('/upload/videoinfo', 'UploadController@videoinfo');
Route::post('/upload/youtube', 'UploadController@youtube');
Route::get('/upload/import', 'UploadController@import');

Route::get('/library', 'HomeController@library');

//ajax stuff


Route::post('/ajax/add_device', 'HomeController@add');
Route::get('/ajax/screens', 'HomeController@screens');
Route::post('/ajax/screen/update/{id}', 'HomeController@updateUserScreen');
Route::post('/ajax/screen/delete_ad/{id}', 'HomeController@deleteAdScreen');
Route::delete('/ajax/screen/{id}', 'HomeController@deleteScreen');
Route::put('/ajax/screen/{id}/activate', 'HomeController@activateScreen');
Route::put('/ajax/screen/{id}/deactivate', 'HomeController@deactivateScreen');
Route::post('/ajax/screen/position/update/{id}', 'HomeController@updateUserScreenPosition');
Route::post('/ajax/screen/mode/update/{id}', 'HomeController@updateUserScreenMode');
Route::post('/ajax/screen/copy/{id}', 'HomeController@copyScreen');
Route::post('/ajax/settings/update', 'HomeController@updateSettings');
Route::post('/ajax/save_time', 'HomeController@saveTime');
Route::get('/ajax/settings', 'HomeController@settings');
Route::get('/ajax/screen/kiosk/{id}/toggle', 'HomeController@toggleKiosk');


Route::post('/ajax/screen/portrait/upload/{id}', 'HomeController@uploadPortraitImage');
Route::post('/ajax/screen/portrait/update_message/{id}', 'HomeController@updatePortraitMessage');


Route::post('/ajax/upload/update/{upload_id}', 'HomeController@updateUploadInfo');
Route::post('/ajax/upload/add_screen/{upload_id}', 'HomeController@addUploadToScreen');
Route::get('/ajax/upload/delete/{upload_id}', 'HomeController@deleteUpload');

Route::resource('users', 'UsersController');

//list stuff
Route::group(['prefix'=>'list'], function(){

	Route::get('/screens', 'ListController@screens');
	Route::get('/screen/media/{id}', 'ListController@screenMedia');
	Route::get('/library', 'ListController@library');

});
